<?php

namespace InspireLabs\WooInpost;

use ConcatPdf;
use InspireLabs\WooInpost\EasyPack;
use Exception;
use InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines;
use InspireLabs\WooInpost\shipx\models\courier_pickup\ShipX_Dispatch_Order_Model;
use Requests_Utility_CaseInsensitiveDictionary;

/**
 * EasyPack Helper
 *
 * @author      WPDesk
 * @category
 * @package     EasyPack
 * @version     2.1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'EasyPack_Helper' ) ) :

	class EasyPack_Helper {

		protected static $instance;

		public function __construct() {
			add_filter( 'query_vars', [ $this, 'query_vars' ] );
			add_action( 'parse_request', [ $this, 'parse_request' ] );

			add_action( 'woocommerce_before_my_account',
				[ $this, 'woocommerce_before_my_account' ] );

			add_filter( 'woocommerce_screen_ids',
				[ $this, 'woocommerce_screen_ids' ] );
		}

		public static function EasyPack_Helper() {
			if ( self::$instance === null ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		public function write_stickers_to_file( $stickers = [] ) {
			$temp_dir = trailingslashit( get_temp_dir() );
			if ( sizeof( $stickers ) == 1 ) {
				$temp_file = tempnam( $temp_dir, 'ep' );
				$fp        = fopen( $temp_file, 'w' );
				fwrite( $fp, $stickers[0] );
				fclose( $fp );
			} else {
				$files = [];
				foreach ( $stickers as $sticker ) {
					$temp_file = tempnam( $temp_dir, 'ep' );
					$fp        = fopen( $temp_file, 'w' );
					fwrite( $fp, $sticker );
					fclose( $fp );
					$files[] = $temp_file;
				}

				$temp_file = tempnam( $temp_dir, 'ep' );
				$pdf       = new ConcatPdf();
				$pdf->setFiles( $files );
				$pdf->concat();
				$pdf->Output( $temp_file, 'F' );
				foreach ( $files as $file ) {
					unlink( $file );
				}
			}

			return $temp_file;
		}

		public function get_file( $file, $file_name, $content_type = '' ) {

			header( 'Content-type: ' . $content_type );
			header( 'Content-Disposition: attachment; filename="' . $file_name
			        . '"' );
			header( 'Content-Transfer-Encoding: binary' );
			header( 'Content-Length: ' . filesize( $file ) );
			header( 'Accept-Ranges: bytes' );

			@readfile( $file );

			unlink( $file );

		}

		public function print_stickers(
			$return_stickers = false,
			$orders = null
		) {
			$ret = [ 'status' => 'ok' ];

			if ( null === $orders ) {
				$orders = $_POST['easypack_parcel'];
			}

			if ( is_numeric( $orders ) && $orders !== 0 ) {
				$orders = [ (int) $orders ];
			}

			$selected_shipments_ids = [];
			$shipment_service       = EasyPack()->get_shipment_service();
			foreach ( $orders as $order ) {
				$selected_shipments_ids[] = $shipment_service
					->get_shipment_by_order_id( $order )
					->getInternalData()
					->getInpostId();
			}


			try {
				if ( true === $return_stickers ) {
					$results
						= EasyPack_API()->customer_shipments_return_labels( $selected_shipments_ids );
				} else {
					$results
						= EasyPack_API()->customer_shipments_labels( $selected_shipments_ids );
				}



				if (!isset($results['headers'])){
					return;
				} else {
					$headers = $results['headers'];
				}

				/**
				 * @var Requests_Utility_CaseInsensitiveDictionary $headers
				 */

				header( sprintf( "Content-type:%s",
					$headers->getAll()['content-type'] ) );

				echo $results['body'];
				die();

			} catch ( Exception $e ) {
				$ret['status']  = 'error';
				$ret['message'] = $e->getMessage();
				wp_die( __( 'Error while creating manifest:  ',
						EasyPack::$text_domain ) . $e->getMessage() );

			}

			/* $stickers = array();

			 foreach ( $parcels as $parcel ) {
				 $order_parcel = explode( '.', $parcel );
				 $api = $order_parcel[0];
				 $order_id = $order_parcel[1];
				 $parcel_id = $order_parcel[2];
				 $easypack_parcels = get_post_meta( $order_id, '_easypack_parcels', true );
				 try {
					 foreach ( $easypack_parcels as $key => $easypack_parcel ) {
						 if ( $api == 'crossborder' ) {
							 if ( $easypack_parcel['crossborder_data']['id'] == $parcel_id ) {
								 $stickers[] = CrossBorder_API()->pdf_label( $easypack_parcel['crossborder_data']['id'], true );
							 }
						 }
						 else {
							 if ( $easypack_parcel['easypack_data']['id'] == $parcel_id ) {
								 if ( $easypack_parcel['easypack_data']['status'] == 'created' ) {
									 $easypack_data = EasyPack_API()->customer_parcel( $easypack_parcel['easypack_data']['id'] );
									 if ( $easypack_data['status'] == 'created' ) {
										 $easypack_data = EasyPack_API()->customer_parcel_pay( $easypack_data['id'] );
										 $easypack_parcels[$key]['easypack_data'] = $easypack_data;
										 $easypack_parcel['easypack_data'] = $easypack_data;
										 update_post_meta( $order_id, '_easypack_parcels', $easypack_parcels );
										 update_post_meta( $order_id, '_easypack_status', 'prepared' );
									 }
								 }
								 $stickers[] = EasyPack_API()->customer_parcel_sticker( $easypack_parcel['easypack_data']['id'] );
							 }
						 }
					 }
				 }
				 catch ( Exception $e ) {
					 $ret['status'] = 'error';
					 $ret['message'] = $e->getMessage();
				 }
				 $tmp_file = $this->write_stickers_to_file( $stickers );
			 }

			 $ret['file'] = basename($tmp_file);

			 echo json_encode( $ret );
			 wp_die();*/
		}


		public function print_dispatch_order( $order_id ) {
			$ret = [ 'status' => 'ok' ];

			try {
				$results = EasyPack_API()->dispatch_order_pdf( $order_id );

				header( sprintf( "Content-type:%s",
					$results['headers']['data']['content-type'] ) );

				echo $results['body'];
				wp_die();

			} catch ( Exception $e ) {
				$ret['status']  = 'error';
				$ret['message'] = $e->getMessage();
				wp_die( __( 'Error while creating dispatch_order:  ',
						EasyPack::$text_domain ) . $e->getMessage() );

			}
		}

		/**
		 * Allow for custom query variables
		 */
		public function query_vars( $query_vars ) {
			$query_vars[] = 'easypack_download';

			return $query_vars;
		}

		/**
		 * Parse the request
		 */
		public function parse_request( &$wp ) {
			if ( array_key_exists( 'easypack_download', $wp->query_vars ) ) {
				if ( isset( $_GET['easypack_parcel_machines_stickers'] )
				     && $_GET['easypack_parcel_machines_stickers'] == '1'
				) {
					EasyPack_Shippng_Parcel_Machines::get_stickers();
				}
				if ( isset( $_GET['easypack_file'] ) ) {
					$temp_dir = trailingslashit( get_temp_dir() );
					$file     = $temp_dir . $_GET['easypack_file'];
					$this->get_file( $file,
						__( 'stickers', EasyPack::$text_domain ) . '_' . time()
						. '.pdf', 'application/pdf' );
				}
				if ( isset( $_GET['easypack_manifest'] ) ) {
					$pickup_service = EasyPack()->get_courier_pickup_service();
					$order
					                = $pickup_service->get_dispatch_order( $_GET['easypack_manifest'] );

					if ( false === $order instanceof
					               ShipX_Dispatch_Order_Model
					) {
						wp_die( __( 'Can not get Dispatch Order from post',
							EasyPack::$text_domain ) );
					}


					$this->print_dispatch_order( $order->getId() );
				}

				exit;
			}
		}

		public function get_geowidget_src( $cod = false, $country = false ) {
			if ( ! $country ) {
				$country = EasyPack_API()->normalize_country_code_for_geowidget(
					EasyPack_API()->getCountry()
				);
			}
			/* RM 11107 */
			if ( EasyPack_API()->is_uk() ) {
				$cod = true;
			}
			$cod_string = '&cod=FALSE';
			if ( $cod ) {
				$cod_string = '&cod=TRUE';
			}

			if ( 'gb' === $country ) {
				$country = 'uk';
			}
			$geowidget_src          = 'https://geowidget-' . $country
			                          . '.easypack24.net/dropdown.php?dropdown_name=parcel_machine_id'
			                          . $cod_string;
			$geowidget_src          = str_replace( '-test-', '-',
				$geowidget_src );
			$easypack_geowidget_url = get_option( 'easypack_geowidget_url',
				'' );
			if ( EasyPack_API()->is_uk()
			     && $easypack_geowidget_url != ''
			) {
				$geowidget_src = trailingslashit( $easypack_geowidget_url )
				                 . 'dropdown.php?dropdown_name=parcel_machine_id'
				                 . $cod_string;
			}

			return 'https://geowidget.easypack24.net/uk/';
		}

		/**
		 * @param string|null $country
		 *
		 * @return string
		 */
		public function get_tracking_url( $country = null ) {
			if ( null === $country ) {
				if ( EasyPack_API()->is_pl() ) {
					return 'https://twoj.inpost.pl/pl/znajdz-przesylke?parcel=';
				}

				return 'https://tracking.inpost.co.uk/';
			}

			if ( EasyPack_API()->normalize_country_code_for_inpost( $country )
			     === EasyPack_API::COUNTRY_PL
			) {
				return 'https://twoj.inpost.pl/pl/znajdz-przesylke?parcel=';
			}

			return 'https://tracking.inpost.co.uk/';
		}

		public function get_order_weight( $order ) {
			$weight = 0;
			if ( sizeof( $order->get_items() ) > 0 ) {
				foreach ( $order->get_items() as $item ) {
					if ( $item['product_id'] > 0 ) {
						$_product = $order->get_product_from_item( $item );
						if ( ! $_product->is_virtual() ) {
							$weight += $_product->get_weight() * $item['qty'];
						}
					}
				}
			}

			return $weight;
		}

		public function get_weight_option( $weight, $options ) {
			$ret     = - 1;
			$options = array_reverse( $options, true );
			foreach ( $options as $val => $option ) {
				if ( floatval( $weight ) <= floatval( $val ) ) {
					$ret = $val;
				}
			}

			return $ret;
		}

		function get_easypack_parcel_data( $parcels, $id ) {
			$ret = false;
			foreach ( $parcels as $parcel ) {
				if ( $parcel['easypack_data']['id'] == $id ) {
					$ret = $parcel['easypack_data'];
				}
			}

			return $ret;
		}

		function get_crossborder_parcel_data( $parcels, $id ) {
			$ret = false;
			foreach ( $parcels as $parcel ) {
				if ( $parcel['crossborder_data']['id'] == $id ) {
					$ret = $parcel['crossborder_data'];
				}
			}

			return $ret;
		}

		function woocommerce_before_my_account() {
			if ( get_option( 'easypack_returns_page' )
			     && trim( get_option( 'easypack_returns_page' ) ) != ''
			) {
				$page = get_page( get_option( 'easypack_returns_page' ) );
				if ( $page ) {
					$img_src = EasyPack()->getPluginImages()
					           . 'logo/small/white.png';
					$args
					         = [
						'returns_page'       => get_page_link( get_option( 'easypack_returns_page' ) ),
						'returns_page_title' => $page->post_title,
						'img_src'            => $img_src,
					];
					wc_get_template( 'myaccount/before-my-account.php', $args,
						'', plugin_dir_path( EasyPack()->getPluginFilePath() )
						    . 'templates/' );
				}
			}
		}

		function woocommerce_screen_ids( $screen_ids ) {
			$screen_ids[] = 'inpost_page_easypack_shipment';

			return $screen_ids;
		}
	}


endif;