<?php

namespace InspireLabs\WooInpost;

class Geowidget_v5 {

	const GEOWIDGET_PRODUCTION_JS_SRC = 'https://geowidget.inpost.pl/inpost-geowidget.js';

	const GEOWIDGET_SANDBOX_JS_SRC = 'https://sandbox-easy-geowidget-sdk.easypack24.net/inpost-geowidget.js';

	const GEOWIDGET_PRODUCTION_CSS_SRC = 'https://geowidget.inpost.pl/inpost-geowidget.css';

	const GEOWIDGET_SANDBOX_CSS_SRC = 'https://sandbox-easy-geowidget-sdk.easypack24.net/inpost-geowidget.css';

	const GEOWIDGET_WIDTH = 800;

	const GEOWIDGET_HEIGHT = 600;

	private $environment;
	private $assets_js_uri;

	public function __construct() {
		$this->environment   = EasyPack::get_environment();
		$this->assets_js_uri = EasyPack::get_assets_js_uri();
	}

	public function init_assets() {
		add_action( 'admin_enqueue_scripts',
			[ $this, 'enqueue_admin_scripts' ], 76 );
		add_action( 'admin_footer', [ $this, 'admin_footer' ], 76 );
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ],
			76 );
		add_action( 'wp_footer', [ $this, 'frontFooter' ], 76 );

	}

	public
	function enqueue_scripts() {
		wp_enqueue_style( 'geowidget-css',
			$this->get_geowidget_css_src() );
	}

	function enqueue_admin_scripts() {
		wp_enqueue_script( 'easypack-admin-geowidget-settings',
			$this->assets_js_uri . 'admin-geowidget-settings.js',
			[ 'jquery' ] );
		wp_localize_script( 'easypack-admin-geowidget-settings',
			'easypackAdminGeowidgetSettings',
			[
				'width'  => self::GEOWIDGET_WIDTH,
				'height' => self::GEOWIDGET_HEIGHT,
				'token'  => $this->get_token(),
				'title'  => __( 'Select parcel locker',
					EasyPack()::getTextDomain() )
			]
		);

		wp_enqueue_style( 'geowidget-css',
			$this->get_geowidget_css_src() );
	}

	/**
	 * @return string
	 */
	private function get_geowidget_js_src(): string {
		return EasyPack::ENVIRONMENT_SANDBOX === $this->environment
			? self::GEOWIDGET_SANDBOX_JS_SRC
			: self::GEOWIDGET_PRODUCTION_JS_SRC;
	}

	/**
	 * @return string
	 */
	private function get_geowidget_css_src(): string {
		return EasyPack::ENVIRONMENT_SANDBOX === $this->environment
			? self::GEOWIDGET_SANDBOX_CSS_SRC
			: self::GEOWIDGET_PRODUCTION_CSS_SRC;
	}

	function admin_footer() {
		printf( '<script async src="%s"></script>',
			$this->get_geowidget_js_src() );
	}

	public function frontFooter() {
		printf( '<script async src="%s"></script>',
			$this->get_geowidget_js_src() );
	}

	public function get_token(): string {
		return EasyPack::ENVIRONMENT_SANDBOX === $this->environment
			? get_option( 'easypack_geowidget_sandbox_token' )
			: get_option( 'easypack_geowidget_production_token' );
	}


	/**
	 * @param string $shipping_method_id
	 *
	 * @return string
	 */
	public function get_pickup_delivery_configuration( string $shipping_method_id
	): string {
		if ( 'config_2' === get_option( 'easypack_delivery_point_configuration' ) ) {
			switch ( $shipping_method_id ) {
				case 'easypack_parcel_machines':
					return 'parcelCollect';

				case 'easypack_parcel_machines_cod':
					return 'parcelCollectPayment';

				case 'easypack_shipping_courier_c2c':
					return 'parcelSend';

			}
		} else {
			switch ( $shipping_method_id ) {
				case 'easypack_parcel_machines':
					return 'parcelCollect';

				case 'easypack_parcel_machines_cod':
					return 'parcelCollectPayment';

				case 'easypack_shipping_courier_c2c':
					return 'parcelSend';

			}
		}
	}


	/**
	 * @return string
	 */
	public function get_pickup_point_configuration(): string {
		return 'parcelSend';
	}
}
