<?php

namespace InspireLabs\WooInpost\admin;

use InspireLabs\WooInpost\EasyPack;
use WC_Shipping;
use WC_Shipping_Method;

class EasyPack_Product_Shipping_Method_Selector {

	const NO_METHODS_SELECTED = 0;

	const META_ID = EasyPack::APP_REFIX . '_shipping_methods_allowed';

	public static $inpost_methods;

	public function handle_product_edit_hooks() {
		add_filter( 'woocommerce_product_data_tabs',
			[ $this, 'filter_woocommerce_product_data_tabs' ], 10, 1 );
		add_action( 'woocommerce_product_data_panels',
			[ $this, 'action_woocommerce_product_data_panels' ], 10, 0 );
		add_action( 'woocommerce_admin_process_product_object',
			[ $this, 'action_woocommerce_admin_process_product_object' ], 10,
			1 );
	}

	/**
	 * @return WC_Shipping_Method[]
	 */
	private function get_inpost_methods(): array {
		return self::$inpost_methods;

		//$methods = WC()->shipping()->load_shipping_methods();
		//$methods = WC()->session->get( 'shipping_methods' );
		$wc_shipping = WC_Shipping::instance();
		$methods     = $wc_shipping->get_shipping_methods();

		$return = array_map( function ( WC_Shipping_Method $method ) {
			if ( get_class( $method ) === 'InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines'
			     || is_subclass_of( get_class( $method ),
					'InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines' ) ) {
				return $method;
			}

			return null;

		}, $methods );

		return array_filter( $return );
	}


	/**
	 * @param int $product_id
	 *
	 * @return array|null
	 */
	private function get_config_by_product_id( int $product_id ): ?array {
		$meta = get_post_meta( $product_id, self::META_ID, true );

		return empty( $meta ) ? null : $meta;
	}

	/**
	 * @param int $product_id
	 *
	 * @return array
	 */
	private function
	get_inpost_methods_by_product_id(
		int $product_id
	): array {
		$return = [];
		
		$config_raw = $this->get_config_by_product_id( $product_id );
		if ( null !== $config_raw && is_array( $config_raw ) ) {
			return $config_raw;
		}

		$all_inpost_methods = $this->get_inpost_methods();
		
		if( ! empty( $all_inpost_methods ) ) {
            foreach ( $all_inpost_methods as $method ) {
                $return[] = $method->id;
            }
        }

		return $return;
	}


	/**
	 * Add custom product setting tab.
	 */
	public function filter_woocommerce_product_data_tabs( $default_tabs ) {
		$default_tabs['custom_tab'] = [
			'label'    => __( 'InPost', EasyPack::$text_domain ),
			'target'   => 'wk_custom_tab_data',
			'priority' => 60,
			'class'    => [],
		];

		return $default_tabs;
	}

	/**
	 * @param array|null $config_by_product
	 * @param $method_id
	 *
	 * @return bool
	 */
	private function is_checked( ?array $config_by_product, $method_id ) {
		if ( empty( $config_by_product ) ) {
			return true;//true is default when config not exists
		}

		if ( in_array( $method_id, $config_by_product ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Contents custom product setting tab.
	 */
	public function action_woocommerce_product_data_panels() {
		global $post;
		echo '<div id="wk_custom_tab_data" class="panel woocommerce_options_panel">';
		$config_by_product = $this->get_config_by_product_id( $post->ID );

		foreach ( $this->get_inpost_methods() as $method ) {
			woocommerce_wp_checkbox( [
				'id'    => $this->get_post_key_from_method_id( $method->id ),
				'label' => $method->get_method_title(),
				'value' => $this->is_checked( $config_by_product,
					$method->id ) ? 'yes' : null,
			] );
		}

		echo '</div>';
	}

	/**
	 * @param string $method_id
	 *
	 * @return string
	 */
	private function get_post_key_from_method_id( string $method_id ): string {
		return '_' . EasyPack::APP_REFIX . '_shipping_method_id_' . $method_id;
	}


	/**
	 * Save the checkbox.
	 */
	public function action_woocommerce_admin_process_product_object( $product
	) {
		$allowed_methods = [];
		foreach ( $this->get_inpost_methods() as $method ) {
			$post_key = $this->get_post_key_from_method_id( $method->id );
			if ( isset( $_POST[ $post_key ] ) && $_POST[ $post_key ] === 'yes' ) {
				$allowed_methods[] = $method->id;
			}
		}

		if ( empty( $allowed_methods ) ) {
			$allowed_methods = self::NO_METHODS_SELECTED;
		}

		$product->update_meta_data( self::META_ID, $allowed_methods );
	}

	public function get_methods_allowed_by_cart( array $contents_of_the_cart
	): array {
		$config_by_product = [];
		
		if( ! empty( $contents_of_the_cart ) ) {
			
			foreach ( $contents_of_the_cart as $cart_item_key => $cart_item ) {
				$product_id                       = $cart_item['product_id'];
				$config_by_product[ $product_id ] = $this->get_inpost_methods_by_product_id( $product_id );
			}

			if ( count( $contents_of_the_cart ) === 1 ) {
				return  $config_by_product[ $product_id ];
			}
		}

		if( ! empty( $config_by_product ) && count( $config_by_product ) > 1 ) {
			return call_user_func_array( 'array_intersect', $config_by_product );
		}
		
		return $config_by_product;
	}

	public function get_methods_disallowed_by_cart( array $contents_of_the_cart
	): array {
		$config_by_product = [];
		foreach ( $contents_of_the_cart as $cart_item_key => $cart_item ) {
			$product_id                       = $cart_item['product_id'];
			$config_by_product[ $product_id ] = $this->get_inpost_methods_by_product_id( $product_id );
		}

		if ( count( $contents_of_the_cart ) === 1 ) {
			return  $config_by_product[ $product_id ];
		}

		return call_user_func_array( 'array_diff', $config_by_product );
	}

}