<?php

namespace InspireLabs\WooInpost\admin;

use InspireLabs\WooInpost\EasyPack;
use Exception;
use InspireLabs\WooInpost\EasyPack_API;
use InspireLabs\WooInpost\shipx\models\courier_pickup\ShipX_Dispatch_Order_Point_Model;
use WC_Admin_Settings;
use WC_Settings_Page;

/**
 * EasyPack General Settings
 *
 * @author      WPDesk
 * @category    Admin
 * @package     EasyPack/Admin
 * @version     2.1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'EasyPack_Settings_General' ) ) :

	/**
	 * EasyPack_Settings_General
	 */
	class EasyPack_Settings_General extends WC_Settings_Page {

		static $prevent_duplicate = [];

		/**
		 * Constructor.
		 */
		public function __construct() {

			$this->id    = 'easypack_general';
			$this->label = __( 'InPost', EasyPack::$text_domain );

			add_filter( 'woocommerce_settings_tabs_array',
				[ $this, 'add_settings_page' ], 20 );
			add_action( 'woocommerce_settings_' . $this->id,
				[ $this, 'output' ] );
			add_action( 'woocommerce_settings_save_' . $this->id,
				[ $this, 'save' ] );

			add_action( 'woocommerce_admin_field_button',
				[ $this, 'woocommerce_admin_field_button' ] );

		}

		public function woocommerce_admin_field_button( $value ) {
			$tooltip_html = '<img class="help_tip" data-tip="'
			                . esc_attr( $value['desc_tip'] ) . '" src="'
			                . WC()->plugin_url()
			                . '/assets/images/help.png" height="16" width="16" />';
			?>
            <tr valign="top">
                <th scope="row" class="titledesc">
                    <label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html( $value['title'] ); ?></label>
					<?php echo $tooltip_html; ?>
                </th>
                <td class="forminp forminp-<?php echo sanitize_title( $value['type'] ) ?>">
                    <button
                            name="<?php echo esc_attr( $value['id'] ); ?>"
                            id="<?php echo esc_attr( $value['id'] ); ?>"
                            type="button"
                            style="<?php echo esc_attr( $value['css'] ); ?>"
                            class="<?php echo esc_attr( $value['class'] ); ?>"
                    />
					<?php echo esc_attr( $value['content'] ); ?></button>
                    &nbsp;<span
                            id="<?php echo esc_attr( $value['id'] ); ?>_message"></span>
					<?php wp_nonce_field( $value['id'],
						'nonce_' . $value['id'] ); ?>
                </td>
            </tr>
			<?php
		}

		/**
		 * Output the settings
		 */
		public function output() {
			$easypack_api_change = get_option( 'easypack_api_error_login', 0 );
			?>
            <input id="easypack_api_change" type="hidden"
                   name="easypack_api_change"
                   value="<?php echo $easypack_api_change; ?>">
            <style>
                .form-table {
                    border-bottom: 1px solid #ccc;
                }
            </style>
			<?php
			$settings = $this->get_settings();
			WC_Admin_Settings::output_fields( $settings );
			?>
            <script type="text/javascript">
                jQuery('#easypack_api_url').closest('tr').css('display', 'none');
                jQuery('#easypack_geowidget_url').closest('tr').css('display', 'none');
                //jQuery('#easypack_crossborder_api_url').closest('tr').css('display','none');
                //jQuery('#easypack_api_country').closest('td').append('<button id="easypack_api_url_button" class="button"><?php _e( 'API URL',
					EasyPack::$text_domain ) ?></butto/>');
                //jQuery('#easypack_default_dispatch_point').closest('td').append('<button id="easypack_default_dispatch_point_add_button" class="button"><?php _e( 'New dispatch point',
					EasyPack::$text_domain ) ?></butto/>');
                //jQuery('#easypack_default_dispatch_point').closest('td').append('<img class="help_tip" data-tip="<?php _e( 'Click Save Changes to add a new Dispatch point.',
					EasyPack::$text_domain ) ?>" src="<?php echo plugins_url( '/woocommerce/assets/images/help.png' ); ?>" height="16" width="16" />');

                jQuery('#easypack_api_url_button').click(function () {
                    if (jQuery('#easypack_api_url').closest('tr').css('display') == 'none') {
                        jQuery('#easypack_api_url').closest('tr').css('display', 'table-row');
                        if (jQuery('#easypack_api_country').val() == 'gb' || jQuery('#easypack_api_country').val() == 'test-gb') {
                            jQuery('#easypack_geowidget_url').closest('tr').css('display', 'table-row');
                        }
                        if (jQuery('#easypack_api_country').val() == 'pl' || jQuery('#easypack_api_country').val() == 'test-pl') {
                            //jQuery('#easypack_crossborder_api_url').closest('tr').css('display','table-row');
                        }
                    } else {
                        jQuery('#easypack_api_url').closest('tr').css('display', 'none');
                        jQuery('#easypack_geowidget_url').closest('tr').css('display', 'none');
                        //jQuery('#easypack_crossborder_api_url').closest('tr').css('display','none');
                    }
                    return false;
                })

                /*jQuery('#easypack_default_dispatch_point_add_button').click(function(){
                    jQuery('#easypack_dispatch_point_name').val('');
                    jQuery('#easypack_dispatch_point_name').focus();
                    jQuery('#easypack_dispatch_point_email').val('');
                    jQuery('#easypack_dispatch_point_phone').val('');
                    jQuery('#easypack_dispatch_point_office_hours').val('');
                    jQuery('#easypack_dispatch_point_street').val('');
                    jQuery('#easypack_dispatch_point_building_no').val('');
                    jQuery('#easypack_dispatch_point_flat_no').val('');
                    jQuery('#easypack_dispatch_point_post_code').val('');
                    jQuery('#easypack_dispatch_point_city').val('');
                    return false;
                })*/
                function easypack_address_fields() {
                    if (jQuery('#easypack_api_country').val() == 'gb' || jQuery('#easypack_api_country').val() == 'test-gb') {
                        //jQuery('#easypack_sender_street').attr('required',false);
                        //jQuery('#easypack_sender_building_no').attr('required',false);
                        jQuery('#easypack_sender_flat_no').attr('required', false);
                        jQuery('#easypack_sender_postal_code').attr('required', false);

                        //jQuery('#easypack_sender_street').closest('tr').css('display','none');
                        //jQuery('#easypack_sender_building_no').closest('tr').css('display','none');
                        jQuery('#easypack_sender_flat_no').closest('tr').css('display', 'none');
                        jQuery('#easypack_sender_postal_code').closest('tr').css('display', 'none');

                        //jQuery('#easypack_sender_flat_no').attr('required',true);
                        //jQuery('#easypack_sender_postal_code').attr('required',true);

                        jQuery('#easypack_sender_flat_no').closest('tr').css('display', 'table-row');
                        jQuery('#easypack_sender_address2').closest('tr').css('display', 'table-row');
                        //jQuery('#easypack_sender_postal_code').closest('tr').css('display','table-row');
                    } else {
                        jQuery('#easypack_sender_street').attr('required', true);
                        jQuery('#easypack_sender_building_no').attr('required', true);
                        jQuery('#easypack_sender_flat_no').attr('required', false);
                        jQuery('#easypack_sender_post_code').attr('required', true);

                        jQuery('#easypack_sender_street').closest('tr').css('display', 'table-row');
                        jQuery('#easypack_sender_building_no').closest('tr').css('display', 'table-row');
                        jQuery('#easypack_sender_flat_no').closest('tr').css('display', 'table-row');
                        jQuery('#easypack_sender_post_code').closest('tr').css('display', 'table-row');

                        jQuery('#easypack_sender_flat_no').attr('required', false);
                        jQuery('#easypack_sender_postal_code').attr('required', false);

                        jQuery('#easypack_sender_flat_no').closest('tr').css('display', 'none');
                        jQuery('#easypack_sender_address2').closest('tr').css('display', 'none');
                        jQuery('#easypack_sender_postal_code').closest('tr').css('display', 'none');
                    }
                }

                function easypack_returns() {
                    if (jQuery('#easypack_api_country').val() == 'gb' || jQuery('#easypack_api_country').val() == 'test-gb') {
                        jQuery('#easypack_returns_page').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_returns_page').closest('table').css('display', 'none');
                    } else {
                        jQuery('#easypack_returns_page').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_returns_page').closest('table').css('display', 'table');
                    }
                }

                function easypack_send_options() {
                    if (/* jQuery('#easypack_api_country').val() == 'ca' || jQuery('#easypack_api_country').val() == 'test-ca'
					|| */ jQuery('#easypack_api_country').val() == 'pl' || jQuery('#easypack_api_country').val() == 'test-pl'
                    ) {
                        jQuery('#easypack_default_send_method').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_default_send_method').closest('table').css('display', 'table');
                        //jQuery('#easypack_default_machine_id').attr('required',true);
                    } else {
                        jQuery('#easypack_default_send_method').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_default_send_method').closest('table').css('display', 'none');
                        jQuery('#easypack_default_machine_id').attr('required', false);
                    }
                }

                jQuery('#easypack_api_country').change(function () {
                });

                function easypack_country_change() {
                    if (jQuery('#easypack_api_country').val() == '--') {
                        jQuery('#easypack_login').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_login').closest('table').css('display', 'none');
                        jQuery('#easypack_tax_status').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_tax_status').closest('table').css('display', 'none');
                        jQuery('#easypack_returns_page').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_returns_page').closest('table').css('display', 'none');
                        jQuery('#easypack_default_send_method').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_default_send_method').closest('table').css('display', 'none');
                        jQuery('#easypack_sender_first_name').closest('table').prev().css('display', 'none');
                        jQuery('#easypack_sender_first_name').closest('table').css('display', 'none');
                        jQuery('.button-primary').attr('disabled', true);
                    } else {
                        jQuery('#easypack_login').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_login').closest('table').css('display', 'table');
                        jQuery('#easypack_tax_status').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_tax_status').closest('table').css('display', 'table');
                        jQuery('#easypack_returns_page').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_returns_page').closest('table').css('display', 'table');
                        jQuery('#easypack_default_send_method').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_default_send_method').closest('table').css('display', 'table');
                        jQuery('#easypack_sender_first_name').closest('table').prev().css('display', 'block');
                        jQuery('#easypack_sender_first_name').closest('table').css('display', 'table');
                        jQuery('.button-primary').attr('disabled', false);
                        easypack_address_fields();
                        easypack_returns();
                        easypack_send_options();
                    }
                    if (jQuery('#easypack_api_country').val() == 'pl' || jQuery('#easypack_api_country').val() == 'test-pl') {
                        jQuery('#easypack_dispatch_point_name').attr('required', true);
                        jQuery('#easypack_dispatch_point_email').attr('required', true);
                        jQuery('#easypack_dispatch_point_phone').attr('required', true);
                        jQuery('#easypack_dispatch_point_office_hours').attr('required', false);
                        jQuery('#easypack_dispatch_point_street').attr('required', true);
                        jQuery('#easypack_dispatch_point_building_no').attr('required', true);
                        jQuery('#easypack_dispatch_point_flat_no').attr('required', false);
                        jQuery('#easypack_dispatch_point_post_code').attr('required', true);
                        jQuery('#easypack_dispatch_point_city').attr('required', true);
                    } else {
                        jQuery('#easypack_dispatch_point_name').attr('required', false);
                        jQuery('#easypack_dispatch_point_email').attr('required', false);
                        jQuery('#easypack_dispatch_point_phone').attr('required', false);
                        jQuery('#easypack_dispatch_point_office_hours').attr('required', false);
                        jQuery('#easypack_dispatch_point_street').attr('required', false);
                        jQuery('#easypack_dispatch_point_building_no').attr('required', false);
                        jQuery('#easypack_dispatch_point_flat_no').attr('required', false);
                        jQuery('#easypack_dispatch_point_post_code').attr('required', false);
                        jQuery('#easypack_dispatch_point_city').attr('required', false);
                        jQuery('#easypack_default_machine_id').attr('required', false);
                    }
                    if (jQuery('#easypack_api_country').val() == 'pl' || jQuery('#easypack_api_country').val() == 'test-pl') {
                        if (jQuery('#easypack_api_country').val() == 'pl') {
                            //jQuery('#easypack_crossborder_api_url').val('https://api-xborder-inpost.sheepla.com');
                        } else {
                            //jQuery('#easypack_crossborder_api_url').val('https://test-api-xborder-inpost.sheepla.com');
                        }
                    }
                }

                var api_country = jQuery('#easypack_api_country').val();
                jQuery('#easypack_api_country').change(function () {
                    if (api_country != jQuery('#easypack_api_country').val()) {
                        if (api_country != '--' && !confirm("<?php _e( 'Are you sure to change the country?',
							EasyPack::$text_domain ); ?>")) {
                            jQuery('#easypack_api_country').val(api_country);
//						jQuery('#easypack_api_country').change();
                        }
                        jQuery('#easypack_api_change').val('1');
                        api_country = jQuery('#easypack_api_country').val();
                        easypack_country_change();
                    }
                })
                jQuery('#easypack_login').change(function () {
                    jQuery('#easypack_api_change').val('1');
                })
                jQuery('#easypack_organization_id').change(function () {
                    jQuery('#easypack_api_change').val('1');
                })
                jQuery('#easypack_api_environment').change(function () {
                    jQuery('#easypack_api_change').val('1');
                })
                jQuery('#easypack_token').change(function () {
                    jQuery('#easypack_api_change').val('1');
                })
                jQuery('#easypack_login').keyup(function () {
                    if (easypack_login != jQuery('#easypack_login').val()) {
                        jQuery('#easypack_api_change').val('1');
                    }
                })
                jQuery('#easypack_token').keyup(function () {
                    if (easypack_token != jQuery('#easypack_token').val()) {
                        jQuery('#easypack_api_change').val('1');
                    }
                })
                var easypack_token = jQuery('#easypack_token').val();
                var easypack_login = jQuery('#easypack_login').val();
                easypack_country_change();
                jQuery(document).ready(function () {
                    easypack_country_change();
                });
                //			easypack_address_fields();
                //			easypack_send_options();
            </script>
			<?php
		}


		/**
		 * @param ShipX_Dispatch_Order_Point_Model[] $points
		 *
		 * @return array
		 */
		public function getDispathPointsOptions( $points ) {
			$return = [];

			foreach ( $points as $point ) {
				$return[ $point->getId() ] = $point->getName();
			}

			return $return;
		}

		/**
		 * Get settings array
		 *
		 * @return array
		 * @throws Exception
		 */
		public function get_settings() {
			$send_methods = [
				'parcel_machine' => __( 'Parcel Locker',
					EasyPack::$text_domain ),
				'courier'        => __( 'Courier', EasyPack::$text_domain ),
			];

			$settings = [

                [
                    'title' => __( 'Help', EasyPack::$text_domain ),
                    'type'  => 'title',
                    'desc'  => __( 'Documentation InPost:  ', EasyPack::$text_domain )
                        . '<a href="https://dokumentacja-inpost.atlassian.net/wiki/spaces/PL/pages/61833233/WooCommerce" target="_blank">'
                        . __( 'How to set up plugin', EasyPack::$text_domain ) . '</a>',
                    'id'    => 'help_options',
                ],

				[
					'title' => __( 'Country', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'country_options',
				],

				[
					'title'    => __( 'Country', EasyPack::$text_domain ),
					'id'       => 'easypack_api_country',
					'default'  => '--',
					'type'     => 'select',
					//					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width: 300px;',
					'desc_tip' => null,
					'options'  => [
						'--' => __( 'Select country', EasyPack::$text_domain ),
						'gb' => __( 'UK', EasyPack::$text_domain ),
						'pl' => __( 'Poland', EasyPack::$text_domain ),
					],
				],

				[ 'type' => 'sectionend', 'id' => 'country_options' ],

				[
					'title' => __( 'Logging in', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'general_options',
				],

				[
					'title'             => __( 'API URL',
						EasyPack::$text_domain ),
					'id'                => 'easypack_api_url',
					'css'               => 'min-width:300px;',
					'default'           => 'https://sandbox-api-shipx-pl.easypack24.net',
					'type'              => 'text',
					'desc_tip'          => false,
					'class'             => 'easypack-api-url',
					'custom_attributes' => [ 'required' => 'required' ],
				],
				[
					'title'             => __( 'Organization ID',
						EasyPack::$text_domain ),
					'id'                => 'easypack_organization_id',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'             => __( 'Token',
						EasyPack::$text_domain ),
					'id'                => 'easypack_token',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'    => __( 'Geowidget production token',
						EasyPack::$text_domain ),
					'id'       => 'easypack_geowidget_production_token',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
				],
				[
					'title'    => __( 'Geowidget sandbox token',
						EasyPack::$text_domain ),
					'id'       => 'easypack_geowidget_sandbox_token',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
				],

				[
					'title'   => __( 'API type', EasyPack::$text_domain ),
					//'id'       => 'easypack_geowidget_api_endpoint',
					'id'      => 'easypack_api_environment',
					'default' => 'production',
					'type'    => 'select',
					//					'class'    => 'wc-enhanced-select',
					'css'     => 'min-width: 300px;',
					//'desc_tip' => __( 'To edit click API URL', EasyPack::$text_domain ),
					'options' => [
						'production' => __( 'Production',
							EasyPack::$text_domain ),
						'sandbox'    => __( 'Sandbox', EasyPack::$text_domain ),
					],
				],

				[
					'title'    => __( 'Szybkie zwroty - link',
						EasyPack::$text_domain ),
					'id'       => 'easypack_fast_return',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
					'class'    => 'easypack-api-url',
				],

				[ 'type' => 'sectionend', 'id' => 'general_options' ],

				[
					'title' => __( 'Insurance', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'insurance_options',
				],

				[
					'title'             => __( 'Default insurance amount',
						EasyPack::$text_domain ),
					'id'                => 'easypack_insurance_amount_default',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => false ],
				],


				[ 'type' => 'sectionend', 'id' => 'general_options' ],


				[
					'title' => __( 'Send options', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'send_options',
				],

				[
					'title'    => __( 'Default package size',
						EasyPack::$text_domain ),
					'id'       => 'easypack_default_package_size',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width: 300px;',
					'desc_tip' => false,
					'default'  => 'A',
					'options'  => EasyPack()->get_package_sizes(),
				],

				[
					'title'    => __( 'Default send method',
						EasyPack::$text_domain ),
					'id'       => 'easypack_default_send_method',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width: 300px;',
					'desc_tip' => false,
					'default'  => 'parcel_machine',
					'options'  => $send_methods,
				],

				[
					'title'    => __( 'Parcel locker label format',
						EasyPack::$text_domain ),
					'id'       => 'easypack_label_format',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width: 300px;',
					'desc_tip' => false,
					'default'  => 'A6',
					'options'  => [
						'A6' => 'A6',
						'A4' => 'A4',
					],
				],

				[
					'title'    => __( 'Default send parcel locker',
						EasyPack::$text_domain ),
					'id'       => 'easypack_default_machine_id',
					'type'     => 'text',
					'class'    => 'settings-geowidget',
					'css'      => 'min-width: 300px;',
					'desc_tip' => false,
					'default'  => '',
					'options'  => [],
					'custom_attributes' => [ 'data-geowidget_config' => 'parcelSend' ],
				],

				[ 'type' => 'sectionend', 'id' => 'send_options' ],

				[
					'title' => __( 'Dispatch point', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'dispatch_point_options',
				],


				$this->output_dispath_point_module(),
				[
					'type' => 'hidden',
					'id'   => EasyPack::APP_REFIX . '_dpoint_selected',
				],

				[ 'type' => 'sectionend', 'id' => 'dispatch_point_options' ],


				[
					'title' => __( 'Sender', EasyPack::$text_domain ),
					'type'  => 'title',
					'desc'  => '',
					'id'    => 'sender_options',
				],

				[
					'title'             => __( 'First Name',
						EasyPack::$text_domain ),
					'id'                => 'easypack_sender_first_name',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'             => __( 'Last Name',
						EasyPack::$text_domain ),
					'id'                => 'easypack_sender_last_name',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'    => __( 'Company Name', EasyPack::$text_domain ),
					'id'       => 'easypack_sender_company_name',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
				],

				[
					'title'    => __( 'Street', EasyPack::$text_domain ),
					'id'       => 'easypack_sender_street',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
					//					'custom_attributes' => array( 'required' => 'required' ),
				],

				[
					'title'    => __( 'Building no', EasyPack::$text_domain ),
					'id'       => 'easypack_sender_building_no',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
					//					'custom_attributes' => array( 'required' => 'required' ),
				],

				/*array(
					'title'    	=> __( 'Flat no', EasyPack::$text_domain ),
					'id'       	=> 'easypack_sender_flat_no',
					'css'      	=> 'min-width:300px;',
					'default'  	=> '',
					'type'     	=> 'text',
					'desc_tip' 	=>  false,
				),*/

				[
					'title'   => __( 'Post code', EasyPack::$text_domain ),
					'id'      => 'easypack_sender_post_code',
					'css'     => 'min-width:300px;',
					'default' => '',
					'type'    => 'text',
					//					'desc_tip' 	=>  __( 'Example: 01-011', EasyPack::$text_domain ),
					//					'custom_attributes' => array( 'required' => 'required' ),
				],


				[
					'title'    => __( 'Address 1', EasyPack::$text_domain ),
					'id'       => 'easypack_sender_flat_no',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
					//					'custom_attributes' => array( 'required' => 'required' ),
				],

				[
					'title'    => __( 'Address 2', EasyPack::$text_domain ),
					'id'       => 'easypack_sender_address2',
					'css'      => 'min-width:300px;',
					'default'  => '',
					'type'     => 'text',
					'desc_tip' => false,
					//					'custom_attributes' => array( 'required' => 'required' ),
				],

				[
					'title'   => __( 'Post code', EasyPack::$text_domain ),
					'id'      => 'easypack_sender_postal_code',
					'css'     => 'min-width:300px;',
					'default' => '',
					'type'    => 'text',
					//					'desc_tip' 	=>  __( 'Example: 01-011', EasyPack::$text_domain ),
					//					'custom_attributes' => array( 'required' => 'required' ),
				],

				[
					'title'             => __( 'City', EasyPack::$text_domain ),
					'id'                => 'easypack_sender_city',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'             => __( 'Email',
						EasyPack::$text_domain ),
					'id'                => 'easypack_sender_email',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'email',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				[
					'title'             => __( 'Phone',
						EasyPack::$text_domain ),
					'id'                => 'easypack_sender_phone',
					'css'               => 'min-width:300px;',
					'default'           => '',
					'type'              => 'text',
					'desc_tip'          => false,
					'custom_attributes' => [ 'required' => 'required' ],
				],

				/*[
					'title'    => __( 'Delivery point configuration',
						EasyPack::$text_domain ),
					'id'       => 'easypack_delivery_point_configuration',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width: 300px;',
					'desc_tip' => false,
					'default'  => 'config_1',
					'options'  => [
                            'config_1' => __( 'Configuration 1',
	                            EasyPack::$text_domain ),
                            'config_]2' => __( 'Configuration 2',
	                            EasyPack::$text_domain ),
                    ],
				],*/

				[ 'type' => 'sectionend', 'id' => 'sender_options' ],
			];

			return $settings;
		}

		/**
		 * @param string|int $key
		 * @param array $array
		 *
		 * @return mixed
		 */
		private function get_from_array( $key, $array ) {
			return isset( $array[ $key ] ) ? $array[ $key ] : null;
		}

		/**
		 * Save settings
		 *
		 * @throws Exception
		 */
		public function save() {
			$settings = $this->get_settings();
			WC_Admin_Settings::save_fields( $settings );
			EasyPack_API()->clear_cache();

			$easypack_api_change = $_REQUEST['easypack_api_change'];

			if ( $easypack_api_change == '1'
			     || '1' === get_option( 'easypack_api_login_error' ) ) {

				$ping = EasyPack_API()->ping();

			}


			if ( $easypack_api_change == '0' ) {
				$args              = [];
				$args['first_name']
				                   = get_option( 'easypack_sender_first_name' );
				$args['last_name'] = get_option( 'easypack_sender_last_name' );
				$args['company_name']
				                   = get_option( 'easypack_sender_company_name' );
				$args['phone']     = get_option( 'easypack_sender_phone' );
				$args['email']     = get_option( 'easypack_sender_email' );
				$args['default_machine_id']
				                   = get_option( 'easypack_default_machine_id' );
				$args['address']   = [];


				if ( EasyPack_API()->api_country()
				     !== EasyPack_API::COUNTRY_PL
				) {
					unset( $args['default_machine_id'] );
				}
				if ( isset ( $args['default_machine_id'] )
				     && $args['default_machine_id'] == ''
				) {
					unset( $args['default_machine_id'] );
				}

			}
		}

		/**
		 * @param int $type
		 *
		 * @return array[]
		 * @throws Exception
		 */
		private function output_dispath_point_module(): array {
			add_action( 'woocommerce_admin_field_manage_dispath_points_module',
				function ( array $data ) {
					if ( isset( self::$prevent_duplicate['manage_dispath_points_module'] )
					     && self::$prevent_duplicate['manage_dispath_points_module']
					) {
						return;
					}

					$saved_points = get_option( EasyPack::APP_REFIX
					                            . '_dpoint' );

					// var_dump($saved_points);die;

					/* if ( isset( self::$prevent_duplicate[ $type ] ) && self::$prevent_duplicate[ $type ] ) {
						 return;
					 }*/
					?>
                    <tr valign="top">

                        <th scope="row" class="titledesc">
                            <label for="<?php echo esc_attr( $custom_button_id ); ?>">
								<?php echo esc_attr( $data['name'] ); ?></label>
                        </th>
                        <td class="forminp forminp-text">

                            <table id="<?php echo EasyPack::APP_REFIX ?>_dpoint-cell">
                                <thead>
                                <th><?php _e( 'Selected',
										EasyPack::$text_domain ) ?></th>
                                <th><?php _e( 'Street',
										EasyPack::$text_domain ) ?></th>
                                <th><?php _e( 'Building number',
										EasyPack::$text_domain ) ?></th>
                                <th><?php _e( 'Postal code',
										EasyPack::$text_domain ) ?></th>
                                <th><?php _e( 'City',
										EasyPack::$text_domain ) ?></th>
                                </thead>

								<?php
								if ( is_array( $saved_points )
								     && ! empty( $saved_points
								                 && isset( $saved_points['street'] ) )
								) {

									/*var_dump(get_option( EasyPack::APP_REFIX
									            . '_dpoint_selected'));*/

									/*var_dump(get_option( EasyPack::APP_REFIX
														 . '_dpoint_selected' ));die;*/

									$dpoints
										= is_array( $saved_points['street'] )
										? $saved_points['street']
										: [ 0 => $saved_points['street'] ];

									foreach (
										$dpoints as $i => $v
									) {

										$id = $i;
										?>

                                        <tr id="<?php echo EasyPack::APP_REFIX ?>_dpoint_<?php echo $i ?>"
                                            class="<?php echo EasyPack::APP_REFIX ?>_dpoint-cell-wraper">
                                            <td>
                                                <input type="radio"
                                                       value="<?php echo $i ?>"
													<?php echo $i
													           === (int) get_option( EasyPack::APP_REFIX
													                                 . '_dpoint_selected' )
														? 'checked' : '' ?>
                                                       class="<?php echo EasyPack::APP_REFIX ?>_dpoint_selected"
                                                       name="<?php echo EasyPack::APP_REFIX ?>_dpoint_selected">
                                                <input type="button"
                                                       value="X"
                                                       class="<?php echo EasyPack::APP_REFIX ?>_dpoint-remove"
                                                       data-remove="<?php echo EasyPack::APP_REFIX ?>_dpoint_<?php echo $i ?>">

                                            </td>
                                            <td>
                                                <input
                                                        class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                        value="<?php echo ! empty( $saved_points['street'] ) ? $saved_points['street'][ $i ] : '' ?>"
                                                        name="<?php echo EasyPack::APP_REFIX ?>_dpoint[street][]"
                                                        id="<?php echo EasyPack::APP_REFIX ?>_dpoint_street">
                                            </td>
                                            <td>
                                                <input
                                                        class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                        value="<?php echo ! empty( $saved_points['building_number'] ) ? $saved_points['building_number'][ $i ] : '' ?>"
                                                        name="<?php echo EasyPack::APP_REFIX ?>_dpoint[building_number][]"
                                                        id="<?php echo EasyPack::APP_REFIX ?>_dpoint_building_number">
                                            </td>
                                            <td>
                                                <input
                                                        class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                        value="<?php echo ! empty( $saved_points['postal_code'] ) ? $saved_points['postal_code'][ $i ] : '' ?>"
                                                        name="<?php echo EasyPack::APP_REFIX ?>_dpoint[postal_code][]"
                                                        id="<?php echo EasyPack::APP_REFIX ?>_dpoint_postal_code">
                                            </td>
                                            <td>
                                                <input
                                                        class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                        value="<?php echo ! empty( $saved_points['city'] ) ? $saved_points['city'][ $i ] : '' ?>"
                                                        name="<?php echo EasyPack::APP_REFIX ?>_dpoint[city][]"
                                                        id="<?php echo EasyPack::APP_REFIX ?>_dpoint_city">
                                            </td>
                                        </tr>
										<?php
										//$i ++;
									}


								} else {
									$k = 0;
									?>
                                    <tr id="<?php echo EasyPack::APP_REFIX ?>_dpoint_<?php echo $k ?>"
                                        class="<?php echo EasyPack::APP_REFIX ?>_dpoint-cell-wraper">
                                        <td>
                                            <input type="radio"
                                                   value="<?php echo $k ?>"
												<?php echo $k
												           === (int) get_option( EasyPack::APP_REFIX
												                                 . '_dpoint_selected' )
													? 'checked' : '' ?>
                                                   class="<?php echo EasyPack::APP_REFIX ?>_dpoint_selected"
                                                   name="<?php echo EasyPack::APP_REFIX ?>_dpoint_selected">
                                            <input type="button"
                                                   value="X"
                                                   class="<?php echo EasyPack::APP_REFIX ?>_dpoint-remove"
                                                   data-remove="<?php echo EasyPack::APP_REFIX ?>_dpoint_<?php echo $k ?>">

                                        </td>
                                        <td>
                                            <input
                                                    class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                    value=""
                                                    name="<?php echo EasyPack::APP_REFIX ?>_dpoint[street]"
                                                    id="<?php echo EasyPack::APP_REFIX ?>_dpoint_street">
                                        </td>
                                        <td>
                                            <input
                                                    class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                    value=""
                                                    name="<?php echo EasyPack::APP_REFIX ?>_dpoint[building_number]"
                                                    id="<?php echo EasyPack::APP_REFIX ?>_dpoint_building_number">
                                        </td>
                                        <td>
                                            <input
                                                    class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                    value=""
                                                    name="<?php echo EasyPack::APP_REFIX ?>_dpoint[postal_code]"
                                                    id="<?php echo EasyPack::APP_REFIX ?>_dpoint_postal_code">
                                        </td>
                                        <td>
                                            <input
                                                    class="<?php echo EasyPack::APP_REFIX ?>_dpoint"
                                                    value=""
                                                    name="<?php echo EasyPack::APP_REFIX ?>_dpoint_city[]"
                                                    id="<?php echo EasyPack::APP_REFIX ?>_dpoint_city">
                                        </td>
                                    </tr>
									<?php
								} ?>

                            </table>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <button id="<?php echo EasyPack::APP_REFIX ?>_dpoint_add"
                                    value="Add dispath point"><?php _e( 'Add dispath point',
									EasyPack::$text_domain ) ?>
                            </button>
                        </td>
                    </tr>
					<?php

					self::$prevent_duplicate['manage_dispath_points_module']
						= true;
				},
				10 );


			return
				[
					'name'  => 'Manage dispath points',
					'title' => 'Manage dispath points',
					'type'  => 'manage_dispath_points_module',
					'id'    => EasyPack::APP_REFIX . '_dpoint',

				];
		}
	}

endif;

//return new EasyPack_Settings_General();